@@ -175,29 +175,34 @@ class UploadTmpTable:
 
     def make_tmp(self):
         from kaf_pas.kd.models.uploades_documents import Uploades_documents
-        from kaf_pas.ckk.models.tmp_item_operations_view import Tmp_Item_operations_view
         from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy
+        from kaf_pas.ckk.models.item_document import Item_document
 
         for uploades_document in Uploades_documents.objects.filter(upload_id=self.id):
             for item in Item.objects.filter(document=uploades_document.document):
                 self._rec_perents(child=item, level=1)
+                for item_document in Item_document.objects.filter(document=uploades_document.document):
+                    self._rec_perents(child=item_document.item, level=1)
 
             for lotsman_documents_hierarcy in Lotsman_documents_hierarcy.objects.filter(document=uploades_document.document):
                 for item in Item.objects.filter(lotsman_document=lotsman_documents_hierarcy):
                     self._rec_perents(child=item, level=1)
 
-        print(f'Cnt: {Tmp_Item_operations_view.objects.count()}')
+        # print(f'Cnt: {Tmp_Item_operations_view.objects.count()}')
 
 
 class UploadDocumentTmpTable(UploadTmpTable):
     def make_tmp(self):
-        from kaf_pas.ckk.models.tmp_item_operations_view import Tmp_Item_operations_view
         from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy
+        from kaf_pas.ckk.models.item_document import Item_document
 
         if self.lotsman_document_id == None:
             for item in Item.objects.filter(document_id=self.id):
                 self._rec_perents(child=item, level=1)
 
+            for item_document in Item_document.objects.filter(document_id=self.id):
+                self._rec_perents(child=item_document.item, level=1)
+
             for lotsman_documents_hierarcy in Lotsman_documents_hierarcy.objects.filter(document_id=self.id):
                 for item in Item.objects.filter(lotsman_document=lotsman_documents_hierarcy):
                     self._rec_perents(child=item, level=1)
@@ -205,7 +210,7 @@ class UploadDocumentTmpTable(UploadTmpTable):
             for item in Item.objects.filter(lotsman_document_id=self.lotsman_document_id):
                 self._rec_perents(child=item, level=1)
 
-        print(f'Cnt: {Tmp_Item_operations_view.objects.count()}')
+        # print(f'Cnt: {Tmp_Item_operations_view.objects.count()}')
 
 
 @JsonResponseWithException(printing=False)
