@@ -54,14 +54,14 @@ class DocumentManagerExt:
 
     def make_spw(self, document):
         from kaf_pas.ckk.models.item import Item
-        from kaf_pas.ckk.models.item_line import Item_line
         from kaf_pas.ckk.models.item_refs import Item_refs
-        from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
-        from kaf_pas.kd.models.document_attributes import Document_attributes
         from kaf_pas.kd.models.documents import Documents
-        from kaf_pas.kd.models.spw_attrs import Spw_attrsQuerySet
         from kaf_pas.system.models.contants import Contants
         from kaf_pas.ckk.models.item import ItemManager
+        from kaf_pas.kd.models.spw_attrs import Spw_attrsQuerySet
+        from kaf_pas.kd.models.document_attributes import Document_attributes
+        from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
+        from kaf_pas.ckk.models.item_line import Item_line
         from kaf_pas.ckk.models.item_document import Item_document
 
         top_level = Contants.objects.get(code='audo_top_level')
@@ -100,29 +100,30 @@ class DocumentManagerExt:
                         )
 
                     parent_count = 0
-                    parent = None
-                    cnt = 0
                     for item in item_query:
                         parent_count = Item_refs.objects.filter(parent=item).count()
-                        parent = item
-                        cnt += 1
                         if parent_count > 0:
                             break
 
-                    if parent == None:
-                        parent = Item.objects.create(
-                            STMP_1_id=STMP_1.id if STMP_1 else None,
-                            STMP_2_id=STMP_2.id if STMP_2 else None,
-                            props=props,
-                            document=document,
-                            version=ItemManager.get_verstion(STMP_1=STMP_1, STMP_2=STMP_2, props=Item.props.from_spw) if cnt != 0 else None
-                        )
+                    version = None
+                    if parent_count > 0:
+                        version = ItemManager.get_verstion(STMP_1=STMP_1, STMP_2=STMP_2, props=Item.props.from_spw)
 
-                        Item_refs.objects.create(parent_id=int(top_level.value), child=parent)
-                        self.link_image_to_item(parent)
-                    else:
-                        if parent_count == 0:
-                            Item_document.objects.get_or_create(item=parent, document=document)
+                    while True:
+                        try:
+                            parent = Item.objects.create(
+                                STMP_1_id=STMP_1.id if STMP_1 else None,
+                                STMP_2_id=STMP_2.id if STMP_2 else None,
+                                props=props,
+                                document=document,
+                                version=version
+                            )
+                            break
+                        except IntegrityError:
+                            version += 1
+
+                    Item_refs.objects.create(parent_id=int(top_level.value), child=parent)
+                    self.link_image_to_item(parent)
 
                     if STMP_2 != None:
                         self.rec_image_cwd(parent.id, delete_dbl_spaces(STMP_2.value_str))
@@ -183,6 +184,7 @@ class DocumentManagerExt:
                                         STMP_2__value_str__delete_dbl_spaces=SPC_CLM_MARK,
                                         props=Item.props.relevant
                                     )[0]
+
                                 Item_document.objects.get_or_create(item=child, document=document)
 
                             except IndexError:
@@ -326,8 +328,8 @@ class DocumentManagerExt:
                 while True:
                     try:
                         item = Item.objects.create(
-                            STMP_1_id=STMP_1_attr,
-                            STMP_2_id=STMP_2_attr,
+                            STMP_1=STMP_1_attr,
+                            STMP_2=STMP_2_attr,
                             props=Item.props.relevant | Item.props.from_pdf,
                             document=document,
                             version=version
