@@ -1,8 +1,8 @@
 import logging
 
 from bitfield import BitField
+from django.conf import settings
 from django.db import transaction
-
 from isc_common.bit import TurnBitOn
 from isc_common.common import blinkString
 from isc_common.common.mat_views import refresh_mat_view
@@ -11,6 +11,7 @@ from isc_common.fields.related import ForeignKeyProtect
 from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
 from isc_common.number import DelProps
 from isc_common.progress import managed_progress, ProgressDroped
+
 from kaf_pas.kd.models.pathes import Pathes
 from kaf_pas.production.models import progress_deleted
 
@@ -50,12 +51,12 @@ class UploadesManager(AuditManager):
     def get_queryset(self):
         return UploadesQuerySet(self.model, using=self._db)
 
-    def deleteFromRequest(self, request):
-
-        from isc_common.auth.models.user import User
+    @staticmethod
+    def del_upload(id, user, doc_cnt=0, lotsman_cnt=0, document_id=None, last_step=False):
         from isc_common.models.deleted_progresses import Deleted_progresses
         from kaf_pas.ckk.models.item import Item
         from kaf_pas.ckk.models.item import ItemManager
+        from kaf_pas.ckk.models.item_document import Item_document
         from kaf_pas.ckk.models.item_image_refs import Item_image_refs
         from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
         from kaf_pas.kd.models.documents import Documents
@@ -68,12 +69,163 @@ class UploadesManager(AuditManager):
         from kaf_pas.kd.models.lotsman_documents_hierarcy_refs import Lotsman_documents_hierarcy_refs
         from kaf_pas.kd.models.uploades_documents import Uploades_documents
         from kaf_pas.kd.models.uploades_log import Uploades_log
-        from kaf_pas.production.models.operations_item import Operations_item
+
+        if settings.LOCKS.locked(f'UploadesManager.del_upload_{id}'):
+            return
+
+        settings.LOCKS.acquire(f'UploadesManager.del_upload_{id}')
+
+        if document_id == None:
+            for upload_document in Uploades_documents.objects.filter(upload_id=id):
+                lotsman_cnt += Lotsman_documents_hierarcy.objects.filter(document=upload_document.document).count()
+        else:
+            lotsman_cnt = Lotsman_documents_hierarcy.objects.filter(document_id=document_id).count()
+
+        upload = Uploades.objects.get(id=id)
+
+        if document_id != None:
+            upload_document = Uploades_documents.objects.get(id=document_id)
+
+        qty = Uploades_documents.objects.filter(upload_id=id).count() + lotsman_cnt if document_id == None else 1 + lotsman_cnt
+        with managed_progress(
+                qty=qty,
+                id=f'Remove_upload_{id}',
+                user=user,
+                message=f'Удаление закачки: {upload.path.drived_absolute_path} от {DateTimeToStr(upload.lastmodified)}' if document_id == None else f'Удаление документа: {upload_document.document.file_document}',
+                title='Выполнено',
+                props=TurnBitOn(0, 0)
+        ) as progress:
+            try:
+                res = 0
+                with transaction.atomic():
+                    continue_cnt = 0
+
+                    if document_id == None:
+                        upload_query = Uploades_documents.objects.select_for_update().filter(upload_id=id)
+                    else:
+                        upload_query = Uploades_documents.objects.select_for_update().filter(id=document_id)
+
+                    for upload_document in upload_query:
+
+                        _continue = False
+                        for item in Item.objects.filter(document=upload_document.document):
+                            if ItemManager.delete_recursive(item_id=item.id, delete_lines=True, user=user, props=0, document=upload_document.document, show_progress=True) == 2:
+                                _continue = True
+                                continue_cnt += 1
+                                break
+                            else:
+                                item.delete()
+
+                        if _continue:
+                            continue
+
+                        deleted = Document_attr_cross.objects.filter(document=upload_document.document).delete()
+                        # logger.debug(f'deleted: {deleted}')
+
+                        for documents_history in Documents_history.objects.filter(new_document=upload_document.document):
+                            documents_history.old_document.props |= Documents.props.relevant
+                            documents_history.old_document.save()
+                            deleted = documents_history.delete()
+                            # logger.debug(f'deleted: {deleted}')
+
+                        for thumb in Documents_thumb.objects.filter(document=upload_document.document):
+                            Item_image_refs.objects.filter(thumb=thumb).delete()
+                            deleted = thumb.delete()
+                            # logger.debug(f'deleted: {deleted}')
+
+                        for thumb10 in Documents_thumb10.objects.filter(document=upload_document.document):
+                            Item_image_refs.objects.filter(thumb10=thumb10).delete()
+                            deleted = thumb10.delete()
+                            # logger.debug(f'deleted: {deleted}')
+
+                        for lotsman_document in Lotsman_documents_hierarcy.objects.filter(document=upload_document.document):
+
+                            for item in Item.objects.filter(lotsman_document=lotsman_document):
+                                if ItemManager.delete_recursive(item_id=item.id, delete_lines=True, user=user, props=0, document=lotsman_document, show_progress=True) == 2:
+                                    _continue = True
+                                    continue_cnt += 1
+                                    break
+                                else:
+                                    Item_document.objects.filter(item=item).delete()
+                                    item.delete()
+
+                            if _continue:
+                                break
+
+                            deleted = Lotsman_document_attr_cross.objects.filter(document=lotsman_document).delete()
+                            # logger.debug(f'deleted: {deleted}')
+                            deleted = Lotsman_document_attr_cross.objects.filter(parent_document=lotsman_document).delete()
+                            # logger.debug(f'deleted: {deleted}')
+                            deleted = Lotsman_documents_hierarcy_refs.objects.filter(child=lotsman_document).delete()
+                            # logger.debug(f'deleted: {deleted}')
+                            deleted = Lotsman_documents_hierarcy_refs.objects.filter(parent=lotsman_document).delete()
+                            # logger.debug(f'deleted: {deleted}')
+
+                            for thumb in Documents_thumb.objects.filter(lotsman_document=lotsman_document):
+                                Item_image_refs.objects.filter(thumb=thumb).delete()
+                                deleted = thumb.delete()
+                                # logger.debug(f'deleted: {deleted}')
+
+                            for thumb10 in Documents_thumb10.objects.filter(lotsman_document=lotsman_document):
+                                Item_image_refs.objects.filter(thumb10=thumb10).delete()
+                                deleted = thumb10.delete()
+                                # logger.debug(f'deleted: {deleted}')
+
+                            Lotsman_documents_hierarcy_files.objects.filter(lotsman_document=lotsman_document).delete()
+                            deleted = lotsman_document.delete()
+                            # logger.debug(f'deleted: {deleted}')
+                            lotsman_cnt += 1
+                            if progress.step() != 0:
+                                raise ProgressDroped(progress_deleted)
+
+                        if _continue:
+                            continue
+
+                        # logger.debug(f'deleted: {deleted}')
+                        Uploades_documents.objects.filter(document=upload_document.document).delete()
+                        Item_document.objects.filter(document=upload_document.document).delete()
+                        Documents.objects.filter(id=upload_document.document.id).delete()
+
+                        doc_cnt += 1
+                        # logger.debug('Done.')
+                        if progress.step() != 0:
+                            raise ProgressDroped(progress_deleted)
+
+                    if continue_cnt == 0:
+                        deleted = Uploades_log.objects.filter(upload_id=id).delete()
+                        # logger.debug(f'deleted: {deleted}')
+                        if document_id == None:
+                            res += Uploades.objects.filter(id=id).delete()[0]
+
+                if doc_cnt > 0 and last_step:
+                    progress.setContentsLabel(blinkString(text='Обновление представления "kd_documents_mview"', color='blue'))
+                    refresh_mat_view('kd_documents_mview')
+
+                if lotsman_cnt > 0 and last_step:
+                    progress.setContentsLabel(blinkString(text='Обновление представления "kd_lotsman_documents_hierarcy_mview"', color='blue'))
+                    refresh_mat_view('kd_lotsman_documents_hierarcy_mview')
+
+                settings.LOCKS.release(f'UploadesManager.del_upload_{id}')
+                return res, doc_cnt, lotsman_cnt
+            except ProgressDroped as ex:
+                Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
+                settings.LOCKS.release(f'UploadesManager.del_upload_{id}')
+                raise ex
+
+            except Exception as ex:
+                settings.LOCKS.release(f'UploadesManager.del_upload_{id}')
+                raise ex
+
+    def deleteFromRequest(self, request):
+        from isc_common.auth.models.user import User
 
         ids = request.GET.getlist('ids')
         user = User.objects.get(username=request.GET.get('ws_channel').split('_')[1])
 
         res = 0
+        doc_cnt = 0
+        lotsman_cnt = 0
+
         for i in range(0, len(ids), 2):
             id = ids[i]
             visibleMode = ids[i + 1]
@@ -81,128 +233,15 @@ class UploadesManager(AuditManager):
             if visibleMode != "none":
                 res += super().filter(id=id).soft_delete(visibleMode=visibleMode)
             else:
-                lotsman_cnt = 0
-
-                for upload_document in Uploades_documents.objects.filter(upload_id=id):
-                    lotsman_cnt += Lotsman_documents_hierarcy.objects.filter(document=upload_document.document).count()
-
-                with managed_progress(
-                        qty=Uploades_documents.objects.filter(upload_id=id).count() + lotsman_cnt,
-                        id=f'Remove_upload_{id}',
-                        user=user,
-                        message=f'Удаление закачки: {super().get(id=id).path.drived_absolute_path}',
-                        title='Выполнено',
-                        props=TurnBitOn(0, 0)
-                ) as progress:
-                    try:
-                        with transaction.atomic():
-                            lotsman_cnt = 0
-                            doc_cnt = 0
-                            continue_cnt = 0
-
-                            for upload_document in Uploades_documents.objects.filter(upload_id=id):
-
-                                deleted = Document_attr_cross.objects.filter(document=upload_document.document).delete()
-                                # logger.debug(f'deleted: {deleted}')
-
-                                _continue = False
-                                for item in Item.objects.filter(document=upload_document.document):
-                                    if Operations_item.objects.filter(item=item).count() > 0:
-                                        _continue = True
-                                        continue_cnt += 1
-                                        break
-
-                                    ItemManager.delete_recursive(item_id=item.id, delete_lines=True, user=user, props=0, document=upload_document.document)
-                                    item.delete()
-
-                                if _continue:
-                                    continue
-
-                                for documents_history in Documents_history.objects.filter(new_document=upload_document.document):
-                                    documents_history.old_document.props |= Documents.props.relevant
-                                    documents_history.old_document.save()
-                                    deleted = documents_history.delete()
-                                    # logger.debug(f'deleted: {deleted}')
-
-                                for thumb in Documents_thumb.objects.filter(document=upload_document.document):
-                                    Item_image_refs.objects.filter(thumb=thumb).delete()
-                                    deleted = thumb.delete()
-                                    # logger.debug(f'deleted: {deleted}')
-
-                                for thumb10 in Documents_thumb10.objects.filter(document=upload_document.document):
-                                    Item_image_refs.objects.filter(thumb10=thumb10).delete()
-                                    deleted = thumb10.delete()
-                                    # logger.debug(f'deleted: {deleted}')
-
-                                for lotsman_document in Lotsman_documents_hierarcy.objects.filter(document=upload_document.document):
-
-                                    deleted = Lotsman_document_attr_cross.objects.filter(document=lotsman_document).delete()
-                                    # logger.debug(f'deleted: {deleted}')
-                                    deleted = Lotsman_document_attr_cross.objects.filter(parent_document=lotsman_document).delete()
-                                    # logger.debug(f'deleted: {deleted}')
-                                    deleted = Lotsman_documents_hierarcy_refs.objects.filter(child=lotsman_document).delete()
-                                    # logger.debug(f'deleted: {deleted}')
-                                    deleted = Lotsman_documents_hierarcy_refs.objects.filter(parent=lotsman_document).delete()
-                                    # logger.debug(f'deleted: {deleted}')
-
-                                    for thumb in Documents_thumb.objects.filter(lotsman_document=lotsman_document):
-                                        Item_image_refs.objects.filter(thumb=thumb).delete()
-                                        deleted = thumb.delete()
-                                        # logger.debug(f'deleted: {deleted}')
-
-                                    for thumb10 in Documents_thumb10.objects.filter(lotsman_document=lotsman_document):
-                                        Item_image_refs.objects.filter(thumb10=thumb10).delete()
-                                        deleted = thumb10.delete()
-                                        # logger.debug(f'deleted: {deleted}')
-
-                                    for item in Item.objects.filter(lotsman_document=lotsman_document):
-                                        if Operations_item.objects.filter(item=item).count() > 0:
-                                            _continue = True
-                                            continue_cnt += 1
-                                            break
-
-                                        ItemManager.delete_recursive(item_id=item.id, delete_lines=True, user=user, props=0)
-                                        item.delete()
-
-                                    if _continue:
-                                        break
-
-                                    Lotsman_documents_hierarcy_files.objects.filter(lotsman_document=lotsman_document).delete()
-                                    deleted = lotsman_document.delete()
-                                    # logger.debug(f'deleted: {deleted}')
-                                    lotsman_cnt += 1
-                                    res = progress.step()
-                                    if res != 0:
-                                        raise ProgressDroped(progress_deleted)
-
-                                if _continue:
-                                    continue
-
-                                # logger.debug(f'deleted: {deleted}')
-                                Uploades_documents.objects.filter(document=upload_document.document).delete()
-                                Documents.objects.filter(id=upload_document.document.id).delete()
-
-                                doc_cnt += 1
-                                # logger.debug('Done.')
-                                res = progress.step()
-                                if res != 0:
-                                    raise ProgressDroped(progress_deleted)
-
-                            if continue_cnt == 0:
-                                deleted = Uploades_log.objects.filter(upload_id=id).delete()
-                                # logger.debug(f'deleted: {deleted}')
-                                res += super().filter(id=id).delete()[0]
-
-                        if doc_cnt > 0:
-                            progress.setContentsLabel(blinkString(text='Обновление представления "kd_documents_mview"', color='blue'))
-                            refresh_mat_view('kd_documents_mview')
-
-                        if lotsman_cnt > 0:
-                            progress.setContentsLabel(blinkString(text='Обновление представления "kd_lotsman_documents_hierarcy_mview"', color='blue'))
-                            refresh_mat_view('kd_lotsman_documents_hierarcy_mview')
-                    except ProgressDroped as ex:
-                        Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
-                        raise ex
+                res, _doc_cnt, _lotsman_cnt = UploadesManager.del_upload(
+                    id=id,
+                    user=user,
+                    doc_cnt=doc_cnt,
+                    last_step=len(ids) / 2 == i,
+                    lotsman_cnt=lotsman_cnt
+                )
+                doc_cnt += _doc_cnt
+                lotsman_cnt += _lotsman_cnt
         return res
 
     def confirmationFromRequest(self, request):
